import { EventEmitter, OnDestroy, OnInit } from "@angular/core";
export declare enum Direction {
    NONE = 0,
    NEXT = 1,
    PREV = 2,
}
/**
 * A carousel component is used to browse or navigate through a collection of slides - galleries of images,
 * cards, on-boarding tutorials or page-based interfaces. It can be used as a separate fulls creen element
 * or inside another component
 *
 * ```
 * <igx-carousel (event output bindings) [input bindings] >
 *     ....
 * </igx-carousel>
 * ```
 * @export
 * @class IgxCarousel
 * @implements {OnDestroy}
 */
export declare class IgxCarousel implements OnDestroy {
    /**
     * Sets whether the carousel should loop back to the first slide
     * after reaching the last slide.
     *
     * Default value is true
     *
     * @type {boolean}
     * @memberOf IgxCarousel
     */
    loop: boolean;
    /**
     * Sets whether the carousel can pause the slide transitions.
     *
     * Default value is true
     *
     * @type {boolean}
     * @memberOf IgxCarousel
     */
    pause: boolean;
    interval: number;
    /**
     * Controls whether the carousel should render the left/right navigation buttons.
     *
     * Default value is true
     *
     * @type {boolean}
     * @memberOf IgxCarousel
     */
    navigation: boolean;
    /**
     * An event that is emitted after a slide transition has happened.
     * Provides a reference to the IgxCarousel as an event argument.
     *
     * @type {EventEmitter}
     * @memberOf IgxCarousel
     */
    onSlideChanged: EventEmitter<{}>;
    /**
     * An event that is emitted after a slide has been added to the carousel.
     * Provides a reference to the IgxCarousel as an event argument.
     *
     * @type {EventEmitter}
     * @memberOf IgxCarousel
     */
    onSlideAdded: EventEmitter<{}>;
    /**
     * An event that is emitted after a slide has been removed from the carousel.
     * Provides a reference to the IgxCarousel as an event argument.
     *
     * @type {EventEmitter}
     * @memberOf IgxCarousel
     */
    onSlideRemoved: EventEmitter<{}>;
    /**
     * An event that is emitted after the carousel has been paused.
     * Provides a reference to the IgxCarousel as an event argument.
     *
     * @type {EventEmitter}
     * @memberOf IgxCarousel
     */
    onCarouselPaused: EventEmitter<{}>;
    /**
     * An event that is emitted after the carousel has resumed transitioning between slides.
     * Provides a reference to the IgxCarousel as an event argument.
     *
     * @type {EventEmitter}
     * @memberOf IgxCarousel
     */
    onCarouselPlaying: EventEmitter<{}>;
    /**
     * The collection of slides currently in the carousel
     *
     * @type {Array<Slide>}
     * @memberOf IgxCarousel
     */
    slides: IgxSlide[];
    private _interval;
    private _lastInterval;
    private _playing;
    private _currentSlide;
    private _destroyed;
    private _total;
    ngOnDestroy(): void;
    setAriaLabel(slide: any): string;
    /**
     * The total number of slides in the carousel.
     *
     * @readonly
     * @type {number}
     * @memberOf IgxCarousel
     */
    readonly total: number;
    /**
     * The index of the slide being currently shown.
     *
     * @readonly
     * @type {number}
     * @memberOf IgxCarousel
     */
    readonly current: number;
    /**
     * Returns the state of the carousel - paused or playing.
     *
     * @readonly
     * @type {boolean}
     * @memberOf IgxCarousel
     */
    readonly isPlaying: boolean;
    /**
     * Whether the carousel is destroyed, i.e. `ngOnDestroy` has been called.
     *
     * @readonly
     * @type {boolean}
     * @memberOf Carousel
     */
    readonly isDestroyed: boolean;
    /**
     * Returns the slide corresponding to the provided index or null.
     *
     * @param {number} index
     * @returns {IgxSlide}
     *
     * @memberOf IgxCarousel
     */
    get(index: number): IgxSlide;
    /**
     * Adds a new slide to the carousel.
     *
     * @param {IgxSlide} slide
     *
     * @memberOf IgxCarousel
     */
    add(slide: IgxSlide): void;
    /**
     * Removes the given slide from the carousel.
     *
     * @param {IgxSlide} slide
     * @returns
     *
     * @memberOf IgxCarousel
     */
    remove(slide: IgxSlide): void;
    /**
     * Kicks in a transition for a given slide with a given direction.
     *
     * @param {IgxSlide} slide
     * @param {Direction} [direction=Direction.NONE]
     *
     * @memberOf IgxCarousel
     */
    select(slide: IgxSlide, direction?: Direction): void;
    /**
     * Transitions to the next slide in the carousel.
     *
     * @returns
     *
     * @memberOf IgxCarousel
     */
    next(): void;
    /**
     * Transitions to the previous slide in the carousel.
     *
     * @returns
     *
     * @memberOf IgxCarousel
     */
    prev(): void;
    /**
     * Resumes playing of the carousel if in paused state.
     * No-op otherwise.
     *
     *
     * @memberOf IgxCarousel
     */
    play(): void;
    /**
     * Stops slide transitions if the `pause` options is set to `true`.
     * No-op otherwise.
     *
     *
     * @memberOf IgxCarousel
     */
    stop(): void;
    onKeydown(event: any): void;
    private _moveTo(slide, direction);
    private _resetInterval();
    private _restartInterval();
}
/**
 * A slide component that usually holds an image and/or a caption text.
 * IgxSlide is usually a child component of an IgxCarousel.
 *
 * ```
 * <igx-slide [input bindings] >
 *    <ng-content></ng-content>
 * </igx-slide>
 * ```
 *
 * @export
 * @class IgxSlide
 * @implements {OnInit}
 * @implements {OnDestroy}
 */
export declare class IgxSlide implements OnInit, OnDestroy {
    private carousel;
    /**
     * The current index of the slide inside the carousel
     *
     * @type {number}
     * @memberOf IgxSlide
     */
    index: number;
    /**
     * The target direction for the slide
     *
     * @type {Direction}
     * @memberOf IgxSlide
     */
    direction: Direction;
    active: boolean;
    constructor(carousel: IgxCarousel);
    ngOnInit(): void;
    ngOnDestroy(): void;
}
export declare class IgxCarouselModule {
}
