import { CommonModule } from "@angular/common";
import { ChangeDetectorRef, Component, ComponentFactoryResolver, ContentChildren, ElementRef, EventEmitter, Input, IterableDiffers, NgModule, Output, QueryList, Renderer2, ViewChild, ViewContainerRef } from "@angular/core";
import { FormsModule } from "@angular/forms";
import { DataAccess, DataContainer } from "../data-operations/data-container";
// grid helper components and directives
import { IgxColumnComponent } from "./column.component";
import { IgxCellBodyComponent, IgxCellFooterComponent, IgxCellFooterTemplateDirective, IgxCellHeaderComponent, IgxCellHeaderTemplateDirective, IgxCellTemplateDirective, IgxColumnFilteringComponent, IgxColumnSortingDirective } from "./grid.common";
import { IgxPaginatorComponent } from "./paginator.component";
import { IgxButtonModule } from "../button/button.directive";
import { DataType, DataUtil } from "../data-operations/data-util";
import { SortingDirection } from "../data-operations/sorting-expression.interface";
import { IgxDialog, IgxDialogModule } from "../dialog/dialog.component";
import { IgxDragDropModule } from "../directives/dragdrop.directive";
import { IgxIconModule } from "../icon/icon.component";
import { IgxInput } from "../input/input.directive";
/**
 *
 *
 * @export
 * @class IgxGridComponent
 * @implements {OnInit}
 * @implements {AfterContentInit}
 * @implements {DoCheck}
 * @implements {OnDestroy}
 */
var IgxGridComponent = (function () {
    function IgxGridComponent(_renderer, _elementRef, _detector, _itDiff, _resolver, _viewRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._detector = _detector;
        this._itDiff = _itDiff;
        this._resolver = _resolver;
        this._viewRef = _viewRef;
        this.perPage = 25;
        this.state = {};
        this.id = "igx-grid-" + IgxGridComponent.nextId++;
        this.autoGenerate = false;
        // event emitters
        this.onEditDone = new EventEmitter();
        this.onFilterDone = new EventEmitter();
        this.onSortingDone = new EventEmitter();
        this.onMovingDone = new EventEmitter();
        this.onCellSelection = new EventEmitter();
        this.onRowSelection = new EventEmitter();
        this.onPagingDone = new EventEmitter();
        this.onColumnInit = new EventEmitter();
        this.onBeforeProcess = new EventEmitter();
        this.dataContainer = new DataContainer([]);
        this.columns = [];
        this.filteringExpressions = {};
        this.sortingExpressions = {};
        this._paging = false;
        this.shouldDataBind = false;
        this._differ = this._itDiff.find([]).create(null);
    }
    Object.defineProperty(IgxGridComponent.prototype, "data", {
        get: function () {
            return this._data;
        },
        set: function (value) {
            this._data = value;
            this.dataContainer.data = value;
            this.shouldDataBind = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "paging", {
        get: function () {
            return this._paging;
        },
        set: function (value) {
            this._paging = value;
            if (this._paging) {
                var state = { index: 0, recordsPerPage: this.perPage };
                this.state.paging = DataUtil.mergeDefaultProperties(this.state.paging, state);
            }
            else {
                this.state.paging = null;
            }
            this.shouldDataBind = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "columnsToRender", {
        get: function () {
            return this.columns.filter(function (col) { return !col.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasEditableColumns", {
        get: function () {
            return this.columns.some(function (col) { return col.editable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasSorting", {
        get: function () {
            return this.columns.some(function (col) { return col.sortable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasFiltering", {
        get: function () {
            return this.columns.some(function (col) { return col.filtering; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "editableColumns", {
        get: function () {
            var res = [];
            this.columns.forEach(function (col) {
                if (col.editable) {
                    res.push(col);
                }
            });
            return res;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.ngOnInit = function () {
        this.dataContainer.state = this.state;
        if (this.paging) {
            var state = { index: 0, recordsPerPage: this.perPage };
            this.state.paging = DataUtil.mergeDefaultProperties(this.state.paging, state);
        }
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype.ngAfterContentInit = /**
         * @hidden
         */
    function () {
        var _this = this;
        if (this.data && this.autoGenerate) {
            this.autogenerate();
        }
        else {
            this.checkColumns();
        }
        this.colDiffer = this.cols.changes.subscribe(function (_) {
            _this.checkColumns();
            _this._detector.markForCheck();
        });
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype.ngDoCheck = /**
         * @hidden
         */
    function () {
        var diff = this._differ.diff(this.data);
        if (this.shouldDataBind || diff) {
            var bindingBehavior = {
                process: function (dataContainer) {
                    dataContainer.process();
                }
            };
            this.onBeforeProcess.emit(bindingBehavior);
            bindingBehavior.process(this.dataContainer);
            this.shouldDataBind = false;
        }
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype.ngOnDestroy = /**
         * @hidden
         */
    function () {
        this.colDiffer.unsubscribe();
    };
    IgxGridComponent.prototype.getColumnByIndex = function (index) {
        return this.columns[index];
    };
    IgxGridComponent.prototype.getColumnByField = function (field) {
        return this.columns.find(function (col) { return col.field === field; });
    };
    IgxGridComponent.prototype.formatBody = function (record, column) {
        if (!column.bodyTemplate) {
            return "" + record[column.field];
        }
    };
    IgxGridComponent.prototype.formatHeader = function (column) {
        if (!column.headerTemplate) {
            return "" + (column.header || column.field);
        }
    };
    /**
     * Returns the cell at rowIndex/columnIndex.
     *
     * @param {number} rowIndex
     * @param {number} columnIndex
     * @returns
     *
     * @memberOf IgxGrid
     */
    /**
         * Returns the cell at rowIndex/columnIndex.
         *
         * @param {number} rowIndex
         * @param {number} columnIndex
         * @returns
         *
         * @memberOf IgxGrid
         */
    IgxGridComponent.prototype.getCell = /**
         * Returns the cell at rowIndex/columnIndex.
         *
         * @param {number} rowIndex
         * @param {number} columnIndex
         * @returns
         *
         * @memberOf IgxGrid
         */
    function (rowIndex, columnField) {
        var result = {
            columnField: columnField,
            dataItem: null,
            element: null,
            rowIndex: rowIndex
        };
        var column = this.getColumnByField(columnField);
        var colIndex = this.columnsToRender.indexOf(column);
        var record = this.dataContainer.getRecordByIndex(rowIndex, DataAccess.TransformedData);
        var element = this._elementRef.nativeElement
            .querySelector("td[data-row=\"" + rowIndex + "\"][data-col=\"" + colIndex + "\"]");
        if (record) {
            result.dataItem = record[column.field];
        }
        if (element) {
            result.element = element;
        }
        return result;
    };
    /**
     * Returns
     *
     * @param {number} rowIndex
     * @returns
     *
     * @memberOf IgxGrid
     */
    /**
         * Returns
         *
         * @param {number} rowIndex
         * @returns
         *
         * @memberOf IgxGrid
         */
    IgxGridComponent.prototype.getRow = /**
         * Returns
         *
         * @param {number} rowIndex
         * @returns
         *
         * @memberOf IgxGrid
         */
    function (rowIndex) {
        var _this = this;
        var result = {
            cells: [],
            element: null,
            index: rowIndex,
            record: null
        };
        var colFields = [];
        this.columns.forEach(function (col) { return colFields.push(col.field); });
        var record = this.dataContainer.getRecordByIndex(rowIndex, DataAccess.TransformedData);
        var element = this._elementRef.nativeElement.querySelector("tbody > tr[data-row=\"" + rowIndex + "\"]");
        if (record) {
            result.record = record;
            Object.keys(record).forEach(function (field) {
                if (colFields.indexOf(field) !== -1) {
                    result.cells.push(_this.getCell(rowIndex, field));
                }
            });
        }
        if (element) {
            result.element = element;
        }
        return result;
    };
    /**
     * Focuses the grid cell at position row x column.
     * Fires the onCellSelection event.
     *
     * @param {(number | string)} rowIndex      : the index of the row
     * @param {(number | string)} columnIndex   : the index of the column
     *
     * @memberOf IgxGrid
     */
    /**
         * Focuses the grid cell at position row x column.
         * Fires the onCellSelection event.
         *
         * @param {(number | string)} rowIndex      : the index of the row
         * @param {(number | string)} columnIndex   : the index of the column
         *
         * @memberOf IgxGrid
         */
    IgxGridComponent.prototype.focusCell = /**
         * Focuses the grid cell at position row x column.
         * Fires the onCellSelection event.
         *
         * @param {(number | string)} rowIndex      : the index of the row
         * @param {(number | string)} columnIndex   : the index of the column
         *
         * @memberOf IgxGrid
         */
    function (rowIndex, columnIndex) {
        var cell = this._elementRef.nativeElement
            .querySelector("td[data-row=\"" + rowIndex + "\"][data-col=\"" + columnIndex + "\"]");
        if (cell) {
            cell.focus();
        }
    };
    /**
     * Focuses the grid row at index `index`.
     * Fires the onRowSelection event.
     *
     * @param {(number | string)} index
     *
     * @memberOf IgxGrid
     */
    /**
         * Focuses the grid row at index `index`.
         * Fires the onRowSelection event.
         *
         * @param {(number | string)} index
         *
         * @memberOf IgxGrid
         */
    IgxGridComponent.prototype.focusRow = /**
         * Focuses the grid row at index `index`.
         * Fires the onRowSelection event.
         *
         * @param {(number | string)} index
         *
         * @memberOf IgxGrid
         */
    function (index) {
        var row = this._elementRef.nativeElement.querySelector("tbody > tr[data-row=\"" + index + "\"]");
        if (row) {
            row.focus();
        }
    };
    IgxGridComponent.prototype.filterData = function (searchTerm, column) {
        var _this = this;
        var filterInput = searchTerm;
        if (!searchTerm) {
            delete this.filteringExpressions[column.field];
        }
        else {
            switch (column.dataType) {
                case DataType.Number:
                    filterInput = parseFloat(searchTerm);
                    break;
                case DataType.Boolean:
                    filterInput = Boolean(searchTerm);
                    break;
                default:
                    break;
            }
            var expr = {
                condition: column.filteringCondition,
                fieldName: column.field,
                ignoreCase: column.filteringIgnoreCase,
                searchVal: filterInput
            };
            this.filteringExpressions[column.field] = expr;
        }
        var result = [];
        Object.keys(this.filteringExpressions)
            .forEach(function (expr) { return result.push(_this.filteringExpressions[expr]); });
        this.dataContainer.state.filtering = {
            expressions: result
        };
        if (this.paging) {
            this.dataContainer.state.paging.index = 0;
        }
        this.shouldDataBind = true;
        this.onFilterDone.emit({
            column: column,
            expression: this.filteringExpressions[column.field]
        });
    };
    IgxGridComponent.prototype.addRow = function (rowData, at) {
        this.dataContainer.addRecord(rowData, at);
        this.shouldDataBind = true;
    };
    IgxGridComponent.prototype.deleteRow = function (row) {
        if (typeof row === "object") {
            this.dataContainer.deleteRecord(row);
        }
        else if (typeof row === "number") {
            this.dataContainer.deleteRecordByIndex(row);
        }
        else if (typeof row === "string" && !isNaN(parseInt(row, 10))) {
            this.dataContainer.deleteRecordByIndex(parseInt(row, 10));
        }
        this.shouldDataBind = true;
    };
    IgxGridComponent.prototype.updateRow = function (index, rowData) {
        this.dataContainer.updateRecordByIndex(index, rowData);
        this.onEditDone.emit({ row: this.getRow(index) });
    };
    IgxGridComponent.prototype.updateCell = function (index, columnField, value) {
        var row = this.getRow(index);
        row.index = this.dataContainer.getIndexOfRecord(row.record);
        row.record[columnField] = value;
        this.updateRow(row.index, row.record);
        this.shouldDataBind = true;
    };
    IgxGridComponent.prototype.sortColumn = function (column, direction) {
        var _this = this;
        if (direction === SortingDirection.None) {
            delete this.sortingExpressions[column.field];
        }
        else {
            var expr = {
                dir: direction,
                fieldName: column.field,
                ignoreCase: true
            };
            this.sortingExpressions[column.field] = expr;
        }
        var result = [];
        Object.keys(this.sortingExpressions)
            .forEach(function (expr) { return result.push(_this.sortingExpressions[expr]); });
        this.dataContainer.state.sorting = this.dataContainer.state.sorting || { expressions: [] };
        this.dataContainer.state.sorting.expressions = result;
        this.shouldDataBind = true;
        this.onSortingDone.emit({
            column: column,
            direction: direction,
            expression: this.sortingExpressions[column.field]
        });
    };
    IgxGridComponent.prototype.paginate = function (page) {
        this.paginator.paginate(page);
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype.navigate = /**
         * @hidden
         */
    function (event) {
        var key = event.key;
        var row = event.target.dataset.row;
        var column = event.target.dataset.col;
        row = parseInt(row, 10);
        column = parseInt(column, 10);
        if (key.endsWith("Left")) {
            this.focusCell(row, column - 1);
        }
        else if (key.endsWith("Right")) {
            this.focusCell(row, column + 1);
        }
        else if (key.endsWith("Up")) {
            this.focusCell(row - 1, column);
        }
        else if (key.endsWith("Down")) {
            this.focusCell(row + 1, column);
        }
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype.checkColumns = /**
         * @hidden
         */
    function () {
        var _this = this;
        this.columns = this.cols.toArray();
        this.columns.forEach(function (col, index) {
            col.index = index;
            _this.onColumnInit.emit({ column: col });
        });
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype.onRowFocus = /**
         * @hidden
         */
    function (event, index) {
        var el = event.target;
        this._renderer.setAttribute(el, "aria-selected", "true");
        this._renderer.addClass(el, "igx-grid__tr--selected");
        this.onRowSelection.emit({ row: this.getRow(index) });
    };
    IgxGridComponent.prototype.onRowBlur = function (event) {
        var el = event.target;
        this._renderer.removeAttribute(el, "aria-selected");
        this._renderer.removeClass(el, "igx-grid__tr--selected");
    };
    IgxGridComponent.prototype.onCellFocus = function (event, index, columnField) {
        var el = event.target ? event.target : event;
        this._renderer.setAttribute(el, "aria-selected", "true");
        if (!this.getColumnByField(columnField).bodyTemplate) {
            this._renderer.addClass(el, "igx-grid__td--selected");
            this._renderer.addClass(el.parentElement, "igx-grid__tr--selected");
        }
        this.onCellSelection.emit({ cell: this.getCell(index, columnField) });
    };
    IgxGridComponent.prototype.onCellBlur = function (event) {
        var el = event.target;
        this._renderer.removeAttribute(el, "aria-selected");
        this._renderer.removeClass(el, "igx-grid__td--selected");
        this._renderer.removeClass(el.parentElement, "igx-grid__tr--selected");
    };
    IgxGridComponent.prototype.editCell = function (index, item, row) {
        if (!this.hasEditableColumns) {
            return;
        }
        this._lastRow = row;
        this.selectedRow = {
            index: index,
            item: item,
            row: Object.assign({}, row)
        };
        this.editingModal.open();
    };
    IgxGridComponent.prototype.processFilter = function (event) {
        this.filterData(event.value, event.column);
    };
    IgxGridComponent.prototype._setValue = function (field, event) {
        this.selectedRow.row[field] = event;
    };
    IgxGridComponent.prototype.getSortedExpression = function (column) {
        var dir = 0;
        if (column.sortable && this.state.sorting) {
            var expr = this.state.sorting.expressions
                .filter(function (each) { return each.fieldName === column.field; });
            if (expr && expr.length) {
                dir = expr[0].dir;
            }
        }
        return dir;
    };
    IgxGridComponent.prototype.cancelEdit = function () {
        this.selectedRow.row = this._lastRow;
        if (this.paging) {
            this.selectedRow.index = this.dataContainer.getIndexOfRecord(this._lastRow);
        }
        this.updateRow(this.selectedRow.index, this.selectedRow.row);
        this.shouldDataBind = true;
        this.editingModal.close();
    };
    IgxGridComponent.prototype.saveData = function () {
        if (this.paging) {
            this.selectedRow.index = this.dataContainer.getIndexOfRecord(this._lastRow);
        }
        this.updateRow(this.selectedRow.index, this.selectedRow.row);
        this.shouldDataBind = true;
        this.editingModal.close();
    };
    IgxGridComponent.prototype.moveColumn = function (event) {
        this.columns[event.dragData].index = event.dropData;
        this.columns[event.dropData].index = event.dragData;
        var temp = this.columns[event.dragData];
        this.columns[event.dragData] = this.columns[event.dropData];
        this.columns[event.dropData] = temp;
        this.onMovingDone.emit(event);
    };
    IgxGridComponent.prototype.processSort = function (event) {
        this.sortColumn(event.column, event.direction);
    };
    /**
     * @hidden
     */
    /**
         * @hidden
         */
    IgxGridComponent.prototype._paginate = /**
         * @hidden
         */
    function (event) {
        this.dataContainer.state.paging = {
            index: event.currentPage,
            recordsPerPage: event.perPage
        };
        this.shouldDataBind = true;
        this.onPagingDone.emit(event);
    };
    IgxGridComponent.prototype.autogenerate = function () {
        var _this = this;
        var factory = this._resolver
            .resolveComponentFactory(IgxColumnComponent);
        var colFields = Object.keys(this.data[0]);
        colFields.forEach(function (field, index) {
            var ref = _this._viewRef.createComponent(factory);
            ref.instance.field = field;
            ref.instance.index = index;
            _this.onColumnInit.emit({ column: ref.instance });
            _this.columns.push(ref.instance);
            ref.changeDetectorRef.detectChanges();
        });
    };
    IgxGridComponent.nextId = 1;
    IgxGridComponent.decorators = [
        { type: Component, args: [{
                    selector: "igx-grid",
                    styles: ["\n      .igx-grid{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-flow:column nowrap;flex-flow:column nowrap;width:100%;background:#fff;border-radius:.125em;-webkit-box-shadow:0 1px 5px 0 rgba(0,0,0,0.26),0 2px 2px 0 rgba(0,0,0,0.12),0 3px 1px -2px rgba(0,0,0,0.08);box-shadow:0 1px 5px 0 rgba(0,0,0,0.26),0 2px 2px 0 rgba(0,0,0,0.12),0 3px 1px -2px rgba(0,0,0,0.08);overflow:hidden}.igx-grid__caption{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;font-size:1.25em;line-height:2em;padding:1em 1.5em}.igx-grid__thead,.igx-grid__tfoot{display:block;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.igx-grid__tbody{display:block;-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto;position:relative;overflow-y:auto}.igx-grid__tbody-empty{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;text-align:center}.igx-grid__tr{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-flex:1;-ms-flex:1;flex:1;background:inherit;border-bottom:1px solid rgba(0,0,0,0.12);-webkit-transition:background 0.2s ease;transition:background 0.2s ease}.igx-grid__tr:last-of-type{border-bottom:0}.igx-grid__tr:hover{background:rgba(0,0,0,0.04)}thead:first-of-type:hover{background:none}.igx-grid__tr--selected td{color:#fff;background-color:rgba(0,0,0,0.54)}.igx-grid__th,.igx-grid__td{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-flow:column wrap;flex-flow:column wrap;-webkit-box-flex:1;-ms-flex:1;flex:1;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;outline:none;padding:1.125em 1.5em;font-size:.8125em;line-height:1em;color:rgba(0,0,0,0.74);text-align:left;word-wrap:break-word;overflow:hidden}.igx-grid__th--number,.igx-grid__td--number{text-align:right}.igx-grid__td--selected{color:#fff;background-color:#e41c77 !important}.igx-grid__th{font-size:.75em;color:rgba(0,0,0,0.54);position:relative;overflow:visible}.igx-grid__th-content{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden}.igx-grid__th-content .title{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1}.igx-grid__th-content .icon{font-size:1em;line-height:1em;padding:.5em}.igx-grid__th.asc,.igx-grid__th.desc{color:rgba(0,0,0,0.74)}.igx-grid__th.desc .igx-grid__th-content>.title::before{position:relative;font-family:'Material Icons';content:'\\E5DB';margin-right:8px}.igx-grid__th.asc .igx-grid__th-content>.title::before{position:relative;font-family:'Material Icons';content:'\\E5D8';margin-right:8px}\n    "],
                    template: "\n      <table class=\"igx-grid\" [attr.id]=\"id\" role=\"grid\" [attr.aria-readonly]=\"!hasEditableColumns\">\n          <thead class=\"igx-grid__thead\" role=\"rowgroup\">\n              <tr class=\"igx-grid__tr\" role=\"rowheader\">\n                  <ng-template ngFor let-col [ngForOf]=\"columnsToRender\" let-colIndex=\"index\">\n\n                      <th (onSort)=\"processSort($event)\"\n                          role=\"columnheader\" tabindex=\"0\"\n                          class=\"igx-grid__th\"\n                          [attr.aria-label]=\"col.header || col.field\"\n                          [igxColumnSorting]=\"col\"\n                          [sortDirection]=\"getSortedExpression(col)\"\n                          [style.width]=\"col.width || ''\">\n                          <div class=\"igx-grid__th-content\">\n                              <span class=\"title\">\n                                  <igx-cell-header *ngIf=\"col.headerTemplate\" [column]=\"col\" [colIndex]=\"index\"></igx-cell-header>\n                                  {{ formatHeader(col) }}\n                              </span>\n                              <igx-col-filter\n                                  (onFilter)=\"processFilter($event)\"\n                                  *ngIf=\"col.filtering\"\n                                  [value]=\"filteringExpressions[col.field]?.searchVal || ''\"\n                                  [column]=\"col\">\n                              </igx-col-filter>\n                              <div *ngIf=\"col.movable\" (onDrop)=\"moveColumn($event)\" [igxDraggable]=\"col.index\" [igxDroppable]=\"col.index\">\n                                  <igx-icon name=\"swap_horiz\"></igx-icon>\n                              </div>\n                          </div>\n                      </th>\n                  </ng-template>\n              </tr>\n          </thead>\n          <tbody class=\"igx-grid__tbody\" role=\"rowgroup\" (keydown)=\"navigate($event)\">\n              <div class=\"igx-grid__tbody-empty\" *ngIf=\"!dataContainer.transformedData || !dataContainer.transformedData.length || !columnsToRender || !columnsToRender.length\">\n                  <h4>No data to display</h4>\n              </div>\n              <ng-template ngFor let-rowData [ngForOf]=\"dataContainer.transformedData\" let-rowIndex=\"index\">\n                  <tr class=\"igx-grid__tr\" [attr.data-row]=\"rowIndex\" tabindex=\"0\" role=\"row\" (focus)=\"onRowFocus($event, rowIndex)\" (blur)=\"onRowBlur($event)\">\n                      <ng-template ngFor let-col [ngForOf]=\"columnsToRender\" let-colIndex=\"index\">\n                          <td tabindex=\"0\" role=\"gridcell\" [attr.aria-readonly]=\"!col.editable\" [attr.aria-describedby]=\"id + '_' + col.field\"\n                              [attr.data-row]=\"rowIndex\" [attr.data-col]=\"colIndex\"\n                              class=\"igx-grid__td\"\n                              [style.width]=\"col.width || ''\"\n                              (focus)=\"onCellFocus($event, rowIndex, col.field)\"\n                              (blur)=\"onCellBlur($event)\"\n                              (dblclick)=\"editCell(rowIndex, rowData[col.field], rowData)\"\n                              (keyup.enter)=\"editCell(rowIndex, rowData[col.field], rowData)\">\n                              <igx-cell-body *ngIf=\"col.bodyTemplate\" [column]=\"col\" [row]=\"rowData\" [item]=\"rowData[col.field]\" [rowIndex]=\"rowIndex\"></igx-cell-body>\n                              {{ formatBody(rowData, col) }}\n                          </td>\n                      </ng-template>\n                  </tr>\n              </ng-template>\n          </tbody>\n          <tfoot class=\"igx-grid__tfoot\">\n              <igx-paginator *ngIf=\"paging\"\n                  (onPageChange)=\"_paginate($event)\"\n                  [currentPage]=\"dataContainer.state.paging?.index\"\n                  [perPage]=\"perPage\"\n                  [totalPages]=\"dataContainer.state.paging?.metadata?.countPages\">\n              </igx-paginator>\n          </tfoot>\n          <igx-dialog *ngIf=\"hasEditableColumns\"\n                  title=\"Edit Record\"\n                  leftButtonLabel=\"Cancel\"\n                  (onLeftButtonSelect)=\"editingModal.close()\"\n                  (onRightButtonSelect)=\"saveData()\"\n                  (keyup.enter)=\"saveData()\"\n                  (keyup.escape)=\"cancelEdit()\"\n                  rightButtonLabel=\"Save\"\n                  rightButtonBackgroundColor=\"#00BFB2\"\n                  rightButtonColor=\"white\"\n                  closeOnOutsideSelect=\"true\"\n              >\n              <div class=\"igx-form-group\" *ngFor=\"let col of editableColumns;\">\n                  <input type=\"text\" igxInput [ngModel]=\"selectedRow?.row[col.field]\" (ngModelChange)=\"_setValue(col.field, $event)\" />\n                  <label igxLabel>{{ col.header || col.field }}</label>\n              </div>\n          </igx-dialog>\n      </table>\n    "
                },] },
    ];
    /** @nocollapse */
    IgxGridComponent.ctorParameters = function () { return [
        { type: Renderer2, },
        { type: ElementRef, },
        { type: ChangeDetectorRef, },
        { type: IterableDiffers, },
        { type: ComponentFactoryResolver, },
        { type: ViewContainerRef, },
    ]; };
    IgxGridComponent.propDecorators = {
        "data": [{ type: Input },],
        "perPage": [{ type: Input },],
        "state": [{ type: Input },],
        "paging": [{ type: Input },],
        "id": [{ type: Input },],
        "autoGenerate": [{ type: Input },],
        "editingModal": [{ type: ViewChild, args: [IgxDialog,] },],
        "paginator": [{ type: ViewChild, args: [IgxPaginatorComponent,] },],
        "onEditDone": [{ type: Output },],
        "onFilterDone": [{ type: Output },],
        "onSortingDone": [{ type: Output },],
        "onMovingDone": [{ type: Output },],
        "onCellSelection": [{ type: Output },],
        "onRowSelection": [{ type: Output },],
        "onPagingDone": [{ type: Output },],
        "onColumnInit": [{ type: Output },],
        "onBeforeProcess": [{ type: Output },],
        "cols": [{ type: ContentChildren, args: [IgxColumnComponent,] },],
    };
    return IgxGridComponent;
}());
export { IgxGridComponent };
var GRID_DIRECTIVES = [
    IgxGridComponent,
    IgxColumnComponent,
    IgxColumnFilteringComponent,
    IgxColumnSortingDirective,
    IgxCellTemplateDirective,
    IgxCellFooterTemplateDirective,
    IgxCellBodyComponent,
    IgxCellHeaderComponent,
    IgxCellFooterComponent,
    IgxCellHeaderTemplateDirective,
    IgxPaginatorComponent
];
var IgxGridModule = (function () {
    function IgxGridModule() {
    }
    IgxGridModule.decorators = [
        { type: NgModule, args: [{
                    declarations: GRID_DIRECTIVES,
                    entryComponents: [IgxColumnComponent],
                    exports: GRID_DIRECTIVES,
                    imports: [CommonModule, IgxIconModule, IgxDialogModule, IgxDragDropModule, IgxInput, IgxButtonModule, FormsModule]
                },] },
    ];
    /** @nocollapse */
    IgxGridModule.ctorParameters = function () { return []; };
    return IgxGridModule;
}());
export { IgxGridModule };
//# sourceMappingURL=grid.component.js.map