import { transition, trigger, useAnimation } from "@angular/animations";
import { CommonModule } from "@angular/common";
import { Component, EventEmitter, Input, NgModule, NgZone, Output } from "@angular/core";
import { fadeIn, fadeOut, slideInBottom, slideOutBottom } from "../animations/main";
/**
 * IgxSnackbar provides feedback about an operation by showing brief message at the bottom of the screen on mobile
 * and lower left on larger devices. IgxSnackbar will appear above all
 * other elements on screen and only one can be displayed at a time.
 * ```
 * <igx-snackbar (event output bindings) [input bindings]>
 * </igx-snackbar>
 * ```
 */
var IgxSnackbar = (function () {
    function IgxSnackbar(zone) {
        this.zone = zone;
        /**
             * The IgxSnackbar component visual state state
             * @type {boolean}
             */
        this.isVisible = false;
        /**
             * Sets if the IgxSnackbar component will be hidden after shown
             * Default value is true
             * @type {number}
             */
        this.autoHide = true;
        /**
             * The duration of time span in ms which the IgxSnackbar component will be visible
             * after it is being shown.
             * Default value is 4000
             * @type {number}
             */
        this.displayTime = 4000;
        /**
             * The event that will be thrown when the action is executed,
             * provides reference to the IgxSnackbar component as argument
             * @type {EventEmitter}
             */
        this.onAction = new EventEmitter();
        /**
             * The event that will be thrown when the snackbar animation starts
             * @type {EventEmitter<AnimationTransitionEvent>}
             */
        this.animationStarted = new EventEmitter();
        /**
             * The event that will be thrown when the snackbar animation ends
             * @type {EventEmitter<AnimationTransitionEvent>}
             */
        this.animationDone = new EventEmitter();
    }
    /**
     * Shows the IgxSnackbar component and hides it after some time span
     * if autoHide is enabled
     */
    /**
         * Shows the IgxSnackbar component and hides it after some time span
         * if autoHide is enabled
         */
    IgxSnackbar.prototype.show = /**
         * Shows the IgxSnackbar component and hides it after some time span
         * if autoHide is enabled
         */
    function () {
        var _this = this;
        clearTimeout(this.timeoutId);
        setTimeout(this.timeoutId);
        this.isVisible = true;
        if (this.autoHide) {
            this.timeoutId = setTimeout(function () {
                _this.hide();
            }, this.displayTime);
        }
    };
    /**
     * Hides the IgxSnackbar component
     */
    /**
         * Hides the IgxSnackbar component
         */
    IgxSnackbar.prototype.hide = /**
         * Hides the IgxSnackbar component
         */
    function () {
        this.isVisible = false;
        clearTimeout(this.timeoutId);
    };
    IgxSnackbar.prototype.triggerAction = function () {
        this.onAction.emit(this);
    };
    IgxSnackbar.prototype.snackbarAnimationStarted = function (evt) {
        if (evt.fromState === "void") {
            this.animationStarted.emit(evt);
        }
    };
    IgxSnackbar.prototype.snackbarAnimationDone = function (evt) {
        if (evt.fromState === "show") {
            this.animationDone.emit(evt);
        }
    };
    IgxSnackbar.decorators = [
        { type: Component, args: [{
                    animations: [
                        trigger("slideInOut", [
                            transition("void => *", [
                                useAnimation(slideInBottom, {
                                    params: {
                                        duration: ".35s",
                                        easing: "cubic-bezier(0.0, 0.0, 0.2, 1)",
                                        fromPosition: "translateY(100%)",
                                        toPosition: "translateY(0)"
                                    }
                                })
                            ]),
                            transition("* => void", [
                                useAnimation(slideOutBottom, {
                                    params: {
                                        duration: ".2s",
                                        easing: "cubic-bezier(0.4, 0.0, 1, 1)",
                                        fromPosition: "translateY(0)",
                                        toOpacity: 1,
                                        toPosition: "translateY(100%)"
                                    }
                                })
                            ])
                        ]),
                        trigger("fadeInOut", [
                            transition("void => *", [
                                useAnimation(fadeIn, {
                                    params: {
                                        duration: ".35s",
                                        easing: "ease-out"
                                    }
                                })
                            ]),
                            transition("* => void", [
                                useAnimation(fadeOut, {
                                    params: {
                                        duration: ".2s",
                                        easing: "ease-out"
                                    }
                                })
                            ])
                        ])
                    ],
                    selector: "igx-snackbar",
                    styles: ["\n      .igx-snackbar{display:-webkit-box;display:-ms-flexbox;display:flex;position:relative;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row nowrap;flex-flow:row nowrap;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;min-height:3em;padding:.4375em 1.5em;background:rgba(0,0,0,0.74);color:#fff;-webkit-backface-visibility:hidden;backface-visibility:hidden}.igx-snackbar__message{font-size:.875em}.igx-snackbar__button{background:transparent;color:#e41c77;border:0;line-height:1em;margin-left:1.5em;text-transform:uppercase;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;font-weight:600;-webkit-tap-highlight-color:transparent;outline:none;-webkit-transition:color .2s ease;transition:color .2s ease}.igx-snackbar__button:hover{color:#e41c77}\n    "],
                    template: "\n      <div class=\"igx-snackbar\" *ngIf=\"isVisible\" (@slideInOut.start)=\"snackbarAnimationStarted($event)\" (@slideInOut.done)=\"snackbarAnimationDone($event)\"\n          [@slideInOut]=\"isVisible\">\n          <span class=\"igx-snackbar__message\" [@fadeInOut]=\"isVisible\">{{ message }}</span>\n          <button class=\"igx-snackbar__button\" igxRipple=\"white\" *ngIf=\"actionText\" [@fadeInOut] (click)=\"triggerAction(snackbar)\">\n                  {{ actionText }}\n          </button>\n      </div>\n    "
                },] },
    ];
    /** @nocollapse */
    IgxSnackbar.ctorParameters = function () { return [
        { type: NgZone, },
    ]; };
    IgxSnackbar.propDecorators = {
        "message": [{ type: Input },],
        "isVisible": [{ type: Input },],
        "autoHide": [{ type: Input },],
        "displayTime": [{ type: Input },],
        "actionText": [{ type: Input },],
        "onAction": [{ type: Output },],
        "animationStarted": [{ type: Output },],
        "animationDone": [{ type: Output },],
    };
    return IgxSnackbar;
}());
export { IgxSnackbar };
var IgxSnackbarModule = (function () {
    function IgxSnackbarModule() {
    }
    IgxSnackbarModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxSnackbar],
                    exports: [IgxSnackbar],
                    imports: [CommonModule]
                },] },
    ];
    /** @nocollapse */
    IgxSnackbarModule.ctorParameters = function () { return []; };
    return IgxSnackbarModule;
}());
export { IgxSnackbarModule };
//# sourceMappingURL=snackbar.component.js.map